------------------------------------------------
--                 CT_BarMod                  --
--                                            --
-- Intuitive yet powerful action bar addon,   --
-- featuring per-button positioning as well   --
-- as scaling while retaining the concept of  --
-- grouped buttons and action bars.           --
-- Please do not modify or otherwise          --
-- redistribute this without the consent of   --
-- the CTMod Team. Thank you.                 --
------------------------------------------------

--------------------------------------------
-- Initialization

local _G = getfenv(0);
local module = _G.CT_BarMod;

--------------------------------------------
-- Variables

-- Local Copies
local tremove = tremove;
local type = type;
local GetActionTexture = GetActionTexture;

-- Referenced at multiple locations
local newButtonMeta;
local currentMode;
local currentButtonClass;
local actionButton = { };
local actionButtonList = { };
--local savedButtons;

module.actionButtonList = actionButtonList;

module.maxBarNum = 12;  -- Maximum number of bars allowed

module.controlBarId = 11;  -- id number of the Control Bar
module.controlCancel = 132;  -- action id we're making use of on the control bar to show a cancel button

module.actionBarId = 12;  -- id number of the Action Bar

--------------------------------------------
-- Helpers

local currentHover;

local function updateHover(_, self)
	if ( not self ) then
		self = currentHover;
		if ( not self ) then
			return;
		end
	end
	if ( GetCVar("UberTooltips") == "1" ) then
		-- Note: GameTooltip_SetDefaultAnchor() will set Gametooltip.default to 1.
		-- If we end up not showing a tooltip (because no action associated
		-- with a button), then we want to make sure that the GameTooltip's OnHide
		-- script executes so that it can clear the GameTooltip.default value.
		-- To ensure the OnHide script is executed, set the tooltip to a single
		-- character right before calling GameTooltip:Hide(). That will force the
		-- tooltip to be shown if it wasn't already.
		GameTooltip_SetDefaultAnchor(GameTooltip, self);
	else
		local xthis = self:GetCenter();
		local xui = UIParent:GetCenter();
		if (xthis < xui) then
			GameTooltip:SetOwner(self, "ANCHOR_RIGHT");
		else
			GameTooltip:SetOwner(self, "ANCHOR_LEFT");
		end
	end

	-- Note: If no action is associated with the button then no tooltip
	-- will be shown.
	if ( GameTooltip:SetAction(self.object.actionId) ) then
		currentHover = self;
	elseif (self.object.actionId == module.controlCancel) then
		if (UnitHasVehicleUI("player")) then
			if (CanExitVehicle()) then
				GameTooltip:SetText(LEAVE_VEHICLE or "Leave Vehicle", 1, 1, 1);
				GameTooltip:Show();
				currentHover = self;
			else
				GameTooltip:SetText(LEAVE_VEHICLE or "Leave Vehicle", 0.7, 0.7, 0.7);
				GameTooltip:AddLine("You cannot exit from the current vehicle.", 1, 0.82, 0, true);
				GameTooltip:Show();
				currentHover = self;
			end
		else
			local texture, name, enabled = GetPossessInfo(POSSESS_CANCEL_SLOT or 2);
			if (enabled) then
				GameTooltip:SetText(CANCEL or "Cancel", 1, 1, 1);
				GameTooltip:Show();
				currentHover = self;
			else
				GameTooltip:SetText(CANCEL or "Cancel", 0.7, 0.7, 0.7);
				GameTooltip:AddLine("Unable to cancel. You might be able to cancel a buff instead.", 1, 0.82, 0, true);
				GameTooltip:Show();
				currentHover = self;
			end
		end
	else
		currentHover = nil;
	end
	if (module:getOption("hideTooltip")) then
		-- Before hiding the tooltip, set the tooltip to a character
		-- to ensure the tooltip gets shown. This will ensure that the
		-- GameTooltip's OnHide script will get executed when we hide
		-- the tooltip.
		GameTooltip:SetText(" ");
		GameTooltip:Hide();
	end
end

local actionButtonObjectData;
local function actionButtonObjectSkeleton()
	if ( not actionButtonObjectData ) then
		actionButtonObjectData = {
			-- Background
			"texture#all#i:icon##BACKGROUND",
			
			-- Artwork
			"texture#all#i:flash#hidden#Interface\\Buttons\\UI-QuickslotRed",
			"font#s:36:10#tl:-2:-2#i:hotkey#v:NumberFontNormalSmallGray##r",
			"font#br:-2:2#i:count#v:NumberFontNormal##r",

			-- Overlay
			"font#s:36:10#b:0:2#i:name#v:GameFontHighlightSmallOutline##OVERLAY",
			"texture#s:62:62#mid#i:border#hidden#Interface\\Buttons\\UI-ActionButton-Border#OVERLAY",
			
			-- OnLoad
			["onload"] = function(self)
				local normalTexture = self:CreateTexture(nil, "ARTWORK");
				local pushedTexture = self:CreateTexture(nil, "ARTWORK");
				local highlightTexture = self:CreateTexture(nil, "ARTWORK");
				local checkedTexture = self:CreateTexture(nil, "ARTWORK");
				local cooldown = CreateFrame("Cooldown", nil, self, "CooldownFrameTemplate");
				
				normalTexture:SetWidth(66); normalTexture:SetHeight(66);
				normalTexture:SetPoint("CENTER", self, 0, -1);
				normalTexture:SetTexture("Interface\\Buttons\\UI-Quickslot2");
				normalTexture:SetVertexColor(1, 1, 1, 0.5);
				
				pushedTexture:SetTexture("Interface\\Buttons\\UI-Quickslot-Depress");
				pushedTexture:SetAllPoints(self);
				
				highlightTexture:SetTexture("Interface\\Buttons\\ButtonHilight-Square");
				highlightTexture:SetBlendMode("ADD");
				highlightTexture:SetAllPoints(self);
				
				checkedTexture:SetTexture("Interface\\Buttons\\CheckButtonHilight");
				checkedTexture:SetBlendMode("ADD");
				checkedTexture:SetAllPoints(self);
				
				self:SetNormalTexture(normalTexture);
				self:SetPushedTexture(pushedTexture);
				self:SetHighlightTexture(highlightTexture);
				self:SetCheckedTexture(checkedTexture);
				
				self.FlyoutArrow = self:CreateTexture(nil, "ARTWORK", "ActionBarFlyoutButton-ArrowUp");
				self.FlyoutArrow:SetDrawLayer("ARTWORK", 2);

				self.FlyoutBorder = self:CreateTexture(nil, "ARTWORK", "ActionBarFlyoutButton-IconFrame");
				self.FlyoutBorder:ClearAllPoints();
				self.FlyoutBorder:SetPoint("CENTER", self);
				self.FlyoutBorder:SetDrawLayer("ARTWORK", 1);

				self.FlyoutBorderShadow = self:CreateTexture(nil, "ARTWORK", "ActionBarFlyoutButton-IconShadow");
				self.FlyoutBorderShadow:ClearAllPoints();
				self.FlyoutBorderShadow:SetPoint("CENTER", self);
				self.FlyoutBorderShadow:SetDrawLayer("ARTWORK", 1);

				self.border:SetBlendMode("ADD");
				self.cooldown = cooldown;
				self.normalTexture = normalTexture;
				self.flash:SetDrawLayer("ARTWORK", 1);
				self.hotkey:SetDrawLayer("ARTWORK", 2);
				self.count:SetDrawLayer("ARTWORK", 2);
			end,
			
			-- OnEnter
			["onenter"] = function(self, ...)
				updateHover(nil, self);
				module:schedule(1, true, updateHover);
				self.object:onenter(self, ...);
			end,
			
			-- OnLeave
			["onleave"] = function(self, ...)
				-- Before hiding the tooltip, assign some text to it to ensure that the
				-- tooltip gets shown. We need to do this because we won't have shown a
				-- a tooltip if there was no action associated with the button.
				-- We need to make sure the GameTooltip's OnHide script executes so that
				-- it will reset things like the GameTooltip.default value.
				GameTooltip:SetText(" ");
				GameTooltip:Hide();
				currentHover = nil;
				module:unschedule(updateHover, true);
				self.object:onleave(self, ...);
			end,

			-- OnDragStart
			["ondragstart"] = function(self, ...)
				self.object:ondragstart(self, ...);
			end,
			
			-- OnDragStop
			["ondragstop"] = function(self, ...)
				self.object:ondragstop(self, ...);
			end,
			
			-- OnReceiveDrag
			["onreceivedrag"] = function(self, ...)
				self.object:onreceivedrag(self, ...);
			end,

			-- OnMouseDown
			["onmousedown"] = function(self, ...)
				self.object:onmousedown(self, ...);
			end,
			
			-- OnMouseUp
			["onmouseup"] = function(self, ...)
				self.object:onmouseup(self, ...);
			end,
			
			-- PreClick
			["preclick"] = function(self, ...)
				self.object:preclick(self, ...);
			end,

			-- PostClick
			["postclick"] = function(self, ...)
				self.object:postclick(self, ...);
			end
		};
	end
	return "checkbutton#s:36:36#v:SecureActionButtonTemplate", actionButtonObjectData;
end

local actionButtonObjectPool = { };

local function getActionButton(buttonId)
	local button;
	button = tremove(actionButtonObjectPool);
	if (not button) then
		button = module:getFrame(actionButtonObjectSkeleton, nil, "CT_BarModActionButton" .. buttonId);
	end
	return button
end

local function getActionButtonId()
	local num = #actionButtonList;
	for buttonId = 1, num, 1 do
		if ( not actionButtonList[buttonId] ) then
			return buttonId;
		end
	end
	return num + 1;
end

--------------------------------------------
-- Action Button Class

module.actionButtonClass = actionButton;

-- Create a new object
function actionButton:new(...)
	local button = { };
	setmetatable(button, newButtonMeta);
	button:constructor(...);
	return button;
end

-- Destroy an object
function actionButton:drop()
	self:destructor();
end

-- Constructor, run on object creation
function actionButton:constructor(buttonId, actionId, groupId, count, noInherit, resetMovable)

	if ( noInherit ) then
		return;
	end

	buttonId = buttonId or getActionButtonId();
	actionId = min(132, max(1, actionId or buttonId));

	local button = getActionButton(buttonId);
--	local obj = savedButtons[buttonId];
	
	if ( resetMovable ) then
		module:resetMovable(buttonId);
		self.scale = UIParent:GetScale();
	else
		self.scale = button:GetScale();
	end
	
	self.groupId = groupId or ceil(buttonId / 12);

	self.actionId = actionId;
	self.buttonId = buttonId;
	self.buttonNum = count;
	self.showButton = true;
	self.gridShow = 0;

	module:registerMovable(buttonId, button);
	actionButtonList[buttonId] = self;

	button.object = self;
	button.cooldown.object = self;
	button.action = actionId; -- For use by ActionButton_UpdateFlyout()
	button.ctbarmod = true;  -- Our button.

	button:SetAttribute("gridevent", 0);
	button:SetAttribute("flyoutDirection", module:getOption("flyoutDirection" .. self.buttonId) or "UP");

	self.button = button;
	self.name = button:GetName();
	
	self:savePosition();
	self:setMode(currentMode);
	self:setBinding();

	self:setClickDirection( not not module:getOption("clickDirection") );
	button:RegisterForDrag("LeftButton", "RightButton");
	button:SetAttribute("type", "action");
end

-- Destructor, run on object destruction
function actionButton:destructor(noInherit)

	if ( noInherit ) then
		return;
	end
	
	self.button:Hide();
	actionButtonList[self.buttonId] = nil;
	
	tinsert(self, actionButtonObjectPool);
end

-- General updater
function actionButton:update()
	-- Placeholder for derived classes
end

-- Update texture
function actionButton:updateTexture()
	self.button.icon:SetTexture(GetActionTexture(self.actionId));
end

-- Updates the options table for this button, or creates it
function actionButton:updateOptions()
	local buttonId = self.buttonId;
	local option = module:getOption(buttonId);
	if ( option ) then
		-- Update table
	else
		-- Create new table
	end
end

-- Sets the editing mode
function actionButton:setMode(newMode)
	self:destructor(true);
	setmetatable(self, newButtonMeta);
	self:constructor(nil, nil, nil, nil, true);
	self:update();
end

-- Set if action is triggered on click up or click down.
function actionButton:setClickDirection(down)
	if (down) then
		self.button:RegisterForClicks("AnyDown");
	else
		self.button:RegisterForClicks("AnyUp");
	end
end

-- Change a button's scale
function actionButton:setScale(scale)
	scale = min(max(scale, 0.35), 3);
	self:savePosition();
	self.scale = scale;
	self:updatePosition();
	
	if ( not self.moving ) then
		module:stopMovable(self.buttonId);
	end
end

-- Start moving this button
function actionButton:move()
	self.moving = true;
	module:moveMovable(self.buttonId);
end

-- Stop moving this button
function actionButton:stopMove()
	self.moving = nil;
	module:stopMovable(self.buttonId);
	self:savePosition();
end

-- Save position for this session
function actionButton:savePosition()
	local scale, xPos, yPos = self.scale, self.button:GetCenter();
	self.xPos, self.yPos = xPos*scale, yPos*scale;
	self:updatePosition();
end

-- Update position, takes scale into account
function actionButton:updatePosition()
	local scale, xPos, yPos = self.scale, self.xPos, self.yPos;
	local button = self.button;

	if (button:IsProtected() and InCombatLockdown()) then
		return;
	end
	button:SetScale(scale);
	button:ClearAllPoints();
	button:SetPoint("CENTER", nil, "BOTTOMLEFT", xPos/scale, yPos/scale);
end

-- Set binding depending on saved option
function actionButton:setBinding(binding, delete)
	-- binding = binding or module:getOption("BINDING-" .. self.buttonId);
	if ( binding and not InCombatLockdown()) then
		if (delete) then
			SetBinding(binding, nil);
		else
			SetBindingClick(binding, self.name);
		end
	end
	self:updateBinding();
end

-- Fallback Placeholders
function actionButton:updateBinding() end
function actionButton:ondragstart() end
function actionButton:ondragstop() end
function actionButton:onreceivedrag() end
function actionButton:onmousedown() end
function actionButton:onmouseup() end
function actionButton:postclick() end

--------------------------------------------
-- Action Button List Handler

local lastMethod;
module.actionButtonList = actionButtonList;

local function doMethod(...)
	for buttonId, object in pairs(actionButtonList) do
		object[lastMethod](object, select(2, ...));
	end
end
setmetatable(actionButtonList, { __index =
	function(key, value)
		local obj = currentButtonClass[value];
		if ( type(obj) == "function" ) then
			lastMethod = value;
			return doMethod;
		else
			return obj;
		end
	end
});


--------------------------------------------
-- Event Handlers

local function eventHandler_SlotChanged(event, actionId)
	if (actionId == 0) then
		actionButtonList:update();
	else
		-- local object = actionButtonList[id];
		for buttonId, object in pairs(actionButtonList) do
			if (object and object.actionId == actionId)  then
				object:update();
			end
		end
	end
end

module:regEvent("ACTIONBAR_SLOT_CHANGED", eventHandler_SlotChanged);

--------------------------------------------
-- Mode Handler

function module:setMode(newMode)
	if ( currentMode ~= newMode ) then
		if ( currentMode ) then
			module[currentMode.."Disable"](module);
		end
		newButtonMeta = module[newMode.."ButtonMeta"];
		currentButtonClass = module[newMode.."ButtonClass"];
		actionButtonList:setMode(newMode);
		currentMode = newMode;
		module[newMode.."Enable"](module);
	end
end

--------------------------------------------
-- Group id/number conversion.

-- num == Index into the groupList table.
-- id  == Unique number assigned to the group.
--        This number is used in option and frame names.
--        It is independent of the bar's position in the groupList table.
-- Bar == Name of group.
--        This is what gets displayed on screen.

-- Bar   1 = id  10 = num  1
-- Bar   2 = id   1 = num  2
-- Bar   3 = id   2 = num  3
-- Bar   4 = id   3 = num  4
-- Bar   5 = id   4 = num  5
-- Bar   6 = id   5 = num  6
-- Bar   7 = id   6 = num  7
-- Bar   8 = id   7 = num  8
-- Bar   9 = id   8 = num  9
-- Bar  10 = id   9 = num 10
-- Bar  11 = id  11 = num 11 (control bar)
-- Bar  12 = id  12 = num 12 (action bar)

-- Originally there were 9 bars.
-- - Bar 1 was action ids 13 to 24.
-- - Bar 1 was stored at index 1 of the groupList table.
-- - Bar 2 was action ids 25 to 36.
-- - Bar 2 was stored at index 2 of the groupList table.
-- - Bars 3 through 9 were action ids 37 to 120.
-- - Bars 3 through 9 were stored at index 3 through 9 of the groupList table.
--
-- Later a 10th bar was added which used action ids 1 to 12.
-- - Bar 1 through Bar 9 were renamed Bar 2 through Bar 10.
-- - The new bar was named Bar 1.
-- - Saved options for all bar are stored in a table using a key
--   that consists of an option name and a number. Originally that
--   number was the index used to access the groupList table.
-- - When the new 10th bar was inserted at position 1 in the groupList
--   table, it caused the position of the other bars in the table
--   to shift by 1. This meant that the bar's position in the
--   groupList table could no longer be used to access the saved
--   option.
-- - To allow access to the correct saved option, an id number
--   was assigned to each group. The id number used was equal
--   to the bar's original position in the groupList table.
-- - The new bar which is now in position 1 of the groupList table
--   was assigned id number 10.
--
-- Later an 11th bar was added which used action ids 121 to 132.
-- - The new bar was assigned id number 11.
--
-- Later a 12th bar was added which used variable action id numbers.
-- - The new bar was assigned id number 12.

module.GroupNumToId = function(groupNum)
	-- Convert group number to group id.
	-- Group numbers are temporary values that do not persist beyond the current game session.
	local groupId;
	if (groupNum == 1) then
		-- Id 10 == Num 1.
		groupId = 10;
	else
		if (groupNum <= 10) then
			-- Id 1 to 9 = Num 2 to 10.
			groupId = groupNum - 1;

		else
			-- Id 11 to 12 == Num 11 to 12.
			groupId = groupNum;
		end
	end
	return groupId;
end

module.GroupIdToNum = function(groupId)
	-- Convert group id to group number.
	-- Group id numbers are persistant values that exist beyond the current game session.
	local groupNum;
	if (groupId == 10) then
		-- Num 1 == Id 10.
		groupNum = 1;
	else
		if (groupId <= 9) then
			-- Num 2 to 10 == Id 1 to 9.
			groupNum = groupId + 1;

		else
			-- Num 11 to 12 == Id 11 to 12.
			groupNum = groupId;
		end
	end
	return groupNum;
end
