------------------------------------------------
--               CT_BottomBar                 --
--                                            --
-- Breaks up the main menu bar into pieces,   --
-- allowing you to hide and move the pieces   --
-- independently of each other.               --
-- Please do not modify or otherwise          --
-- redistribute this without the consent of   --
-- the CTMod Team. Thank you.                 --
------------------------------------------------

--------------------------------------------
-- Initialization

local _G = getfenv(0);
local module = _G.CT_BottomBar;

local ctRelativeFrame = module.ctRelativeFrame;
local appliedOptions;

--------------------------------------------
-- Key Ring

local function addon_Update(self)
	-- Update the frame
	-- self == key ring bar object

	local frames = self.frames;
	local obj;

	obj = KeyRingButton;

	-- Anchor the guide frame
	self.helperFrame:ClearAllPoints();
	self.helperFrame:SetPoint("TOPLEFT", obj, 0, 0);
	self.helperFrame:SetPoint("BOTTOMRIGHT", obj);

	-- Set the parent of the objects.
	obj:SetParent(self.frame);

	-- Anchor the objects.
	obj:ClearAllPoints();
	obj:SetPoint("BOTTOMLEFT", self.frame, 0, 0);
end

local function addon_Init(self)
	-- Initialization
	-- self == key ring bar object

	appliedOptions = module.appliedOptions;

	module.ctKeyRing = self;

	self.frame:SetFrameLevel(MainMenuBarArtFrame:GetFrameLevel() + 1);

	local frame = CreateFrame("Frame", "CT_BottomBar_" .. self.frameName .. "_GuideFrame");
	self.helperFrame = frame;

	return true;
end

local function addon_Register()
	module:registerAddon(
		"Key Ring",  -- option name
		"KeyRing",  -- used in frame names
		"Key Ring",  -- shown in options window & tooltips
		"Key",  -- title for horizontal orientation
		nil,  -- title for vertical orientation
		{ "BOTTOMLEFT", ctRelativeFrame, "BOTTOM", 319, 1 },
		{ -- settings
			orientation = "ACROSS",
		},
		addon_Init,
		nil,  -- no post init function
		addon_Update,
		nil,  -- no orientation function
		nil,  -- no enable func
		nil,  -- no disable func
		"helperFrame",
		KeyRingButton
	);
end

module.loadedAddons["Key Ring"] = addon_Register;
