------------------------------------------------
--               CT_BottomBar                 --
--                                            --
-- Breaks up the main menu bar into pieces,   --
-- allowing you to hide and move the pieces   --
-- independently of each other.               --
-- Please do not modify or otherwise          --
-- redistribute this without the consent of   --
-- the CTMod Team. Thank you.                 --
------------------------------------------------

--------------------------------------------
-- Initialization

local _G = getfenv(0);
local module = _G.CT_BottomBar;

local ctRelativeFrame = module.ctRelativeFrame;
local setparent;
local appliedOptions;

--------------------------------------------
-- Reputation Bar

local function addon_AnchorRepToRep(self)
	-- Anchor the reputation bar to the reputation bar's drag frame.
	-- self == rep bar object

	local objRep = ReputationWatchBar;
	local objPlace = self.watchBarFrame;

	setparent(objRep, self.frame);
	objRep:EnableMouse(true);

	-- Clear the points for the guide frame, ReputationWatchBar, and the place holder rep watch bar.
	self.helperFrame:ClearAllPoints();
	objRep:ClearAllPoints();
	objPlace:ClearAllPoints();

	-- Anchor the guide frame around the ReputationWatchBar.
	self.helperFrame:SetPoint("TOPLEFT", objRep, 0, 0);
	self.helperFrame:SetPoint("BOTTOMRIGHT", objRep);

	-- Anchor the ReputationWatchBar to the reputation bar's drag frame.
	objRep:SetPoint("BOTTOMLEFT", self.frame, 0, 0);
end

local function addon_AnchorRepToExp(self)
	-- Anchor the reputation bar to the experience bar's drag frame.
	-- self == rep bar object

	local objRep = ReputationWatchBar;
	local objPlace = self.watchBarFrame;

	setparent(objRep, self.frame);
	objRep:EnableMouse(true);

	-- Clear the points for the guide frame, ReputationWatchBar, and the place holder rep watch bar.
	self.helperFrame:ClearAllPoints();
	objRep:ClearAllPoints();
	objPlace:ClearAllPoints();

	-- Anchor the guide frame around the place holder rep watch bar.
	self.helperFrame:SetPoint("TOPLEFT", objPlace, 0, 0);
	self.helperFrame:SetPoint("BOTTOMRIGHT", objPlace);

	-- Anchor the place holder rep watch bar to the drag frame.
	objPlace:SetPoint("BOTTOMLEFT", self.frame, 0, 0);

	-- Anchor the ReptuationWatchBar to the experience bar's drag frame.
	objRep:SetPoint("BOTTOMLEFT", module.ctExpBar.frame, 0, 2);
end

local function addon_ShowRepBarOnExpBar()
	-- Show the reputation bar instead of the experience bar.

	ReputationWatchStatusBar:SetHeight(12);
	ReputationWatchStatusBarText:SetPoint("CENTER", ReputationWatchBarOverlayFrame, "CENTER", 0, 1);

	addon_AnchorRepToExp(module.ctRepBar);

	ReputationWatchBarTexture0:Hide();
	ReputationWatchBarTexture1:Hide();
	ReputationWatchBarTexture2:Hide();
	ReputationWatchBarTexture3:Hide();

	ReputationXPBarTexture0:Show();
	ReputationXPBarTexture1:Show();
	ReputationXPBarTexture2:Show();
	ReputationXPBarTexture3:Show();

	MainMenuExpBar.pauseUpdates = true;
	MainMenuExpBar:Hide();
end

local function addon_ShowRepBarSeparate()
	-- Show the reputation bar separate from the the experience bar.

	ReputationWatchStatusBar:SetHeight(8);
	ReputationWatchStatusBarText:SetPoint("CENTER", ReputationWatchBarOverlayFrame, "CENTER", 0, 3);

	addon_AnchorRepToRep(module.ctRepBar);

	ReputationWatchBarTexture0:Show();
	ReputationWatchBarTexture1:Show();
	ReputationWatchBarTexture2:Show();
	ReputationWatchBarTexture3:Show();

	ReputationXPBarTexture0:Hide();
	ReputationXPBarTexture1:Hide();
	ReputationXPBarTexture2:Hide();
	ReputationXPBarTexture3:Hide();

	MainMenuExpBar.pauseUpdates = nil;
	MainMenuExpBar:Show();
end

local function addon_Modified_ReputationWatchBar_Update(hideReputationBar, hideExperienceBar, repBarCoverExpBar, repBarHideNoRep, expBarShowMaxLevelBar)

	local name = GetWatchedFactionInfo();
	if (not name) then
		-- Not monitoring a faction.
		ReputationWatchStatusBar:SetMinMaxValues(0, 0);
		ReputationWatchStatusBar:SetValue(0);
		ReputationWatchStatusBarText:SetText("");
	end

	ReputationWatchStatusBar:SetFrameLevel(MainMenuBarArtFrame:GetFrameLevel()-1);

	if (repBarCoverExpBar) then
		-- Show the reputation bar on the experience bar.
		addon_ShowRepBarOnExpBar();
	else
		-- Show the reputation bar separate from the the experience bar.
		addon_ShowRepBarSeparate();
	end

	-- Default ui
	-- 	- when not at max level
	-- 		- enabling "show as experience bar" displays the rep bar above the exp bar
	--		- else it hides rep bar and only shows exp bar
	-- 	- when at max level
	--		- enabling "show as experience bar" replaces the exp bar with the rep bar.
	--		- else it hides the exp bar and shows the max level bar.
	
	local hideExp, hideRep, hideMax;

	if (repBarCoverExpBar) then
		hideExp = true;
	else
		hideExp = false;
	end
	if (hideReputationBar) then
		hideRep = true;
	else
		if (repBarHideNoRep and not name) then
			hideRep = true;
		else
			hideRep = false;
		end
	end
	if (expBarShowMaxLevelBar and hideExp and hideRep) then
		hideMax = false;
	else
		hideMax = true;
	end

	-- Hide or show the rep bar.
	if (hideRep) then
		ReputationWatchBar:Hide();
	else
		ReputationWatchBar:Show();
	end

	-- Hide or show the exp bar.
	if (hideExp) then
		MainMenuExpBar.pauseUpdates = true;
		MainMenuExpBar:Hide();
	else
		MainMenuExpBar.pauseUpdates = nil;
		MainMenuExpBar:Show();
	end

	-- Hide or show the mex level exp bar.
	if (hideMax) then
		MainMenuBarMaxLevelBar:Hide();
	else
		MainMenuBarMaxLevelBar:Show();
	end

	-- Call this function will let Blizzard shift some of their frames to account
	-- for the visibility (or lack of visibility) of the experience and rep bars.
	UIParent_ManageFramePositions();

	-- Update the experience bar.
	module.CT_BottomBar_ExpBar_Update(module.ctExpBar);
end

local function addon_Update(self)
	-- self == rep bar object

	local hideReputationBar, hideExperienceBar, repBarCoverExpBar, repBarHideNoRep, expBarShowMaxLevelBar;

	local hideVehicle = appliedOptions.vehicleHideFrame;
	if (UnitHasVehicleUI("player")) then
		if (not hideVehicle) then
			-- While in a vehicle we want to show the rep and exp bars separately,
			-- since the game anchors the exp bar to the top of the vehicle frame.

			local newRepHideState;
			if (appliedOptions["Reputation Bar"]) then
				-- User always wants the rep bar hidden.
				newRepHideState = true;
			else
				if (appliedOptions.repBarCoverExpBar) then
					-- User was covering the exp bar with the rep bar,
					-- so don't show the rep bar while in the vehicle.
					newRepHideState = true;
				else
					-- The rep bar is being displayed separately.
					-- User wants to see both the rep and exp bars,
					-- so don't hide the rep bar while in the vehicle.
					newRepHideState = false;
				end
			end

			hideReputationBar = newRepHideState;
			hideExperienceBar = true;  -- our exp bar, not the one over the vehicle frame.
			repBarCoverExpBar = false;
			repBarHideNoRep = appliedOptions.repBarHideNoRep;
			expBarShowMaxLevelBar = false;

			addon_Modified_ReputationWatchBar_Update(hideReputationBar, hideExperienceBar, repBarCoverExpBar, repBarHideNoRep, expBarShowMaxLevelBar);
			return;
		end
	elseif (not module.actionBarDisabled) then
		local barType = GetBonusBarOverrideBarType() or "default";
		if (barType ~= "default") then
			-- While using a special bonus action bar frame we want to show the rep and exp bars separately,
			-- since the game anchors the exp bar to the top of the bonus action bar frame.

			if (appliedOptions["Reputation Bar"]) then
				-- User always wants the rep bar hidden.
				newRepHideState = true;
			else
				if (appliedOptions.repBarCoverExpBar) then
					-- User was covering the exp bar with the rep bar,
					-- so don't show the rep bar while using the special bonus bar.
					newRepHideState = true;
				else
					-- The rep bar is being displayed separately.
					-- User wants to see both the rep and exp bars,
					-- so don't hide the rep bar while using the special bonus bar.
					newRepHideState = false;
				end
			end

			hideReputationBar = newRepHideState;
			hideExperienceBar = true;  -- our exp bar, not the one over the special bonus bar.
			repBarCoverExpBar = false;
			repBarHideNoRep = appliedOptions.repBarHideNoRep;
			expBarShowMaxLevelBar = false;

			addon_Modified_ReputationWatchBar_Update(hideReputationBar, hideExperienceBar, repBarCoverExpBar, repBarHideNoRep, expBarShowMaxLevelBar);
			return;
		end
	end

	-- Exp bar is not on a vehicle.
	-- Position the frames where we want them.

	hideReputationBar = appliedOptions["Reputation Bar"];
	hideExperienceBar = appliedOptions["Experience Bar"];
	repBarCoverExpBar = appliedOptions.repBarCoverExpBar;
	repBarHideNoRep = appliedOptions.repBarHideNoRep;
	expBarShowMaxLevelBar = appliedOptions.expBarShowMaxLevelBar;

	addon_Modified_ReputationWatchBar_Update(hideReputationBar, hideExperienceBar, repBarCoverExpBar, repBarHideNoRep, expBarShowMaxLevelBar);
end


local function addon_Hooked_ReputationWatchBar_Update(newLevel)
	-- (hooksecurefunc of ReputationWatchBar_Update in ReputationFrame.lua)
	-- We don't need the newLevel parameter.
	if (module.ctRepBar.isDisabled) then
		return;
	end
	addon_Update(module.ctRepBar);
end

local function addon_Hooked_VehicleMenuBar_MoveMicroButtons(skinName)
	-- (hooksecurefunc of VehicleMenuBar_MoveMicroButtons in VehicleMenuBar.lua)
	module.vehicleSkinName = skinName;
	if (module.ctRepBar.isDisabled) then
		return;
	end
	addon_Update(module.ctRepBar);
end

local function addon_Hooked_AnimTransitionFinished(self)
	-- (hooksecurefunc of ActionBar_AnimTransitionFinished in BonusActionBarFrame.lua)
	if (module.ctRepBar.isDisabled) then
		return;
	end
	module.ctRepBar:toggleVisibility(2, nil);
	module.ctExpBar:toggleVisibility(2, nil);
end

local function addon_Hooked_SetParent(self, parent)
	-- (hooksecurefunc of each bar's SetParent method)
	if (module.ctRepBar.isDisabled) then

		-- If the default action bar is disabled...
		if (module.actionBarDisabled) then

			-- Only do this if the rep bar and the default action bar
			-- have both been disabled.

			-- If the game tries to parent the rep bar to the
			-- BonusActionBarFrame, then it means that the new (WoW 4.0)
			-- larger bonus action bar frame is being displayed.
			--
			-- Because the default action bar has been disabled the
			-- BonusActionBarFrame should not be visible.
			--
			-- We want to disable the mouse in the rep bar to prevent
			-- the mouse from interfering with anything above the rep bar
			-- while it is parented to the BonusActionBarFrame.

			if (parent == BonusActionBarFrame) then
				self:EnableMouse(false);
			else
				self:EnableMouse(true);
			end
		end

		return;
	end
end

local function addon_Init(self)
	-- Initialization
	-- self == rep bar object

	appliedOptions = module.appliedOptions;

	module.ctRepBar = self;

	local frame = CreateFrame("Frame", "CT_BottomBar_" .. self.frameName .. "_GuideFrame");
	self.helperFrame = frame;

	setparent = frame.SetParent;

	frame = CreateFrame("Frame", "CT_BottomBar_" .. self.frameName .. "_WatchBarFrame");
	self.watchBarFrame = frame;

	frame:SetWidth(ReputationWatchBar:GetWidth());
	frame:SetHeight(ReputationWatchBar:GetHeight());
	frame:SetParent(self.frame);

--	addon_AnchorRepToRep(self);

	hooksecurefunc("ReputationWatchBar_Update", addon_Hooked_ReputationWatchBar_Update);
	hooksecurefunc("VehicleMenuBar_MoveMicroButtons", addon_Hooked_VehicleMenuBar_MoveMicroButtons);
	hooksecurefunc("ActionBar_AnimTransitionFinished", addon_Hooked_AnimTransitionFinished);

	return true;
end

local function addon_PostInit(self)
	-- Initialization
	-- self == object

	local frames = self.frames;

	for i = 2, 2 do
		hooksecurefunc(frames[i], "SetParent", addon_Hooked_SetParent);
	end
end

local function addon_Register()
	module:registerAddon(
		"Reputation Bar",  -- option name
		"RepBar",  -- used in frame names
		"Reputation Bar",  -- shown in options window & tooltips
		"Reputation Bar",  -- title for horizontal orientation
		nil,  -- title for vertical orientation
		{ "BOTTOMLEFT", ctRelativeFrame, "BOTTOM", -512, 52 },
		{ -- settings
			orientation = "ACROSS",
			usedOnVehicle = true,
			usedOnBonusBar = true,
		},
		addon_Init,
		addon_PostInit,
		addon_Update,
		nil,  -- no orientation function
		nil,  -- no enable function
		nil,  -- no disable function
		"helperFrame",
		ReputationWatchBar
	);
end

module.loadedAddons["Reputation Bar"] = addon_Register;
