------------------------------------------------
--               CT_BottomBar                 --
--                                            --
-- Breaks up the main menu bar into pieces,   --
-- allowing you to hide and move the pieces   --
-- independently of each other.               --
-- Please do not modify or otherwise          --
-- redistribute this without the consent of   --
-- the CTMod Team. Thank you.                 --
------------------------------------------------

--------------------------------------------
-- Initialization

local _G = getfenv(0);
local module = _G.CT_BottomBar;

local ctRelativeFrame = module.ctRelativeFrame;
local appliedOptions;

-- Data for skins is from VehicleMenuBar.lua
local SkinsData = {
	["Mechanical"] = {
		["PitchUpButton"] = {	--Pitch up button
			height = 36,
			width = 38,
			point = "BOTTOMLEFT",
			xOfs = 146,
			yOfs = 41,
			normalTexture = [[Interface\Vehicles\UI-Vehicles-Button-Pitch-Up]],
			normalTexCoord = { 0.21875, 0.765625, 0.234375, 0.78125 },
			pushedTexture = [[Interface\Vehicles\UI-Vehicles-Button-Pitch-Down]],
			pushedTexCoord = { 0.21875, 0.765625, 0.234375, 0.78125 },
			pitchHidden = 1,
		},
		["PitchDownButton"] = {	--Pitch up button
			height = 36,
			width = 38,
			point = "BOTTOMLEFT",
			xOfs = 146,
			yOfs = 3,
			normalTexture = [[Interface\Vehicles\UI-Vehicles-Button-PitchDown-Up]],
			normalTexCoord = { 0.21875, 0.765625, 0.234375, 0.78125 },
			pushedTexture = [[Interface\Vehicles\UI-Vehicles-Button-PitchDown-Down]],
			pushedTexCoord = { 0.21875, 0.765625, 0.234375, 0.78125 },
			pitchHidden = 1,
		},
		["LeaveButton"] = {	--Leave button
			height = 47,
			width = 50,
			point = "BOTTOMRIGHT",
			xOfs = -148,
			yOfs = 18,
			normalTexture = [[Interface\Vehicles\UI-Vehicles-Button-Exit-Up]],
			normalTexCoord = { 0.140625, 0.859375, 0.140625, 0.859375 },
			pushedTexture = [[Interface\Vehicles\UI-Vehicles-Button-Exit-Down]],
			pushedTexCoord = { 0.140625, 0.859375, 0.140625, 0.859375 },
		},
	},
	["Natural"] = {
		["PitchUpButton"] = {	--Pitch up button
			height = 35,
			width = 34,
			point = "BOTTOMLEFT",
			xOfs = 166,
			yOfs = 41,
			normalTexture = [[Interface\Vehicles\UI-Vehicles-Button-Pitch-Up]],
			normalTexCoord = { 0.21875, 0.765625, 0.234375, 0.78125 },
			pushedTexture = [[Interface\Vehicles\UI-Vehicles-Button-Pitch-Down]],
			pushedTexCoord = { 0.21875, 0.765625, 0.234375, 0.78125 },
			pitchHidden = 1,
		},
		["PitchDownButton"] = {	--Pitch up button
			height = 35,
			width = 34,
			point = "BOTTOMLEFT",
			xOfs = 166,
			yOfs = 3,
			normalTexture = [[Interface\Vehicles\UI-Vehicles-Button-PitchDown-Up]],
			normalTexCoord = { 0.21875, 0.765625, 0.234375, 0.78125 },
			pushedTexture = [[Interface\Vehicles\UI-Vehicles-Button-PitchDown-Down]],
			pushedTexCoord = { 0.21875, 0.765625, 0.234375, 0.78125 },
			pitchHidden = 1,
		},
		["LeaveButton"] = {	--Leave button
			height = 47,
			width = 50,
			point = "BOTTOMRIGHT",
			xOfs = -177,
			yOfs = 15,
			normalTexture = [[Interface\Vehicles\UI-Vehicles-Button-Exit-Up]],
			normalTexCoord = { 0.140625, 0.859375, 0.140625, 0.859375 },
			pushedTexture = [[Interface\Vehicles\UI-Vehicles-Button-Exit-Down]],
			pushedTexCoord = { 0.140625, 0.859375, 0.140625, 0.859375 },
		},
	},
};

--------------------------------------------
-- Vehicle Bar

local function addon_UpdateOrientationNoVehicle(self, orientation)
	-- Update the frame's orientation when not in a vehicle.
	-- self == vehicle bar object
	-- orientation == "ACROSS" or "DOWN"

	local frames = self.frames;
	local offset;
	local obj;
	local attach;

	orientation = orientation or "ACROSS";

	for i = 2, #frames do
		obj = frames[i];
		obj:SetParent(self.frame);
		obj:ClearAllPoints();
	end

	for i = 2, #frames do
		obj = frames[i];
		if (i == 2) then
			obj:SetPoint("BOTTOMLEFT", self.frame);
			offset = 10;
		else
			attach = true;
			if ( 
				(obj == VehicleMenuBarPitchUpButton or obj == VehicleMenuBarPitchDownButton ) and
				(not appliedOptions.vehicleBarAimButtons)
			) then
				-- This is one of the aim buttons, and user does not want to see it
				-- on the vehicle bar, so don't attach it.
				attach = false;
			end
			if (attach) then
				if (orientation == "ACROSS") then
					obj:SetPoint("LEFT", frames[i-1], "RIGHT", offset, 0);
				else
					obj:SetPoint("TOP", frames[i-1], "BOTTOM", 0, -offset);
				end
				offset = 2;
			end
		end
	end
end

local function addon_UpdateOrientationInVehicle(self, orientation)
	-- Anchor the frames when the player has a vehicle UI
	-- self == vehicle bar object
	-- orienation == "ACROSS" or "DOWN"

	local skinName = module.vehicleSkinName;
	if (not skinName) then
		return;
	end
	local skinTable = SkinsData[skinName];
	if (not skinTable) then
		return;
	end

	-- Natural skin:
	-- VehicleMenuBarPitchUpButton, BOTTOMLEFT, VehicleMenuBar, BOTTOMLEFT, 166, 41, Parent=VehicleMenuBar, Lev=8, Mouse=1, H=35, W=34
	-- VehicleMenuBarPitchDownButton, BOTTOMLEFT, VehicleMenuBar, BOTTOMLEFT, 166, 3, Parent=VehicleMenuBar, Lev=8, Mouse=1, H=35, W=34
	-- VehicleMenuBarLeaveButton, BOTTOMRIGHT, VehicleMenuBar, BOTTOMRIGHT, -177, 15, Parent=VehicleMenuBar, Lev=8, Mouse=1, H=47, W=50

	local parent = VehicleMenuBar;
	local pitchVisible = VehicleMenuBar.currPitchVisible;

	for name, framedata in pairs(skinTable) do
		local frame = _G["VehicleMenuBar" .. name];
		frame:SetParent(parent);
		if (framedata.height) then
			frame:SetHeight(framedata.height);
			frame:SetWidth(framedata.width);
		end
		if (framedata.normalTexture) then
			frame:GetNormalTexture():SetTexture(framedata.normalTexture);
			frame:GetNormalTexture():SetTexCoord( unpack(framedata.normalTexCoord) );
		end
		if (framedata.pushedTexture) then
			frame:GetPushedTexture():SetTexture(framedata.pushedTexture);
			frame:GetPushedTexture():SetTexCoord( unpack(framedata.pushedTexCoord) );
		end
		if (framedata.point) then
			frame:ClearAllPoints();
			frame:SetPoint(framedata.point, (framedata.relativeFrame or frame:GetParent()), (framedata.relativePoint or framedata.point), (framedata.xOfs or 0), (framedata.yOfs or 0));
		end
		if ( bit.band((framedata.pitchHidden or 0),(pitchVisible or 0)+1) ~= 0 ) then	--0 = never hide. 1 = hide when no pitch slider 2 = hide when pitch slider
			frame:Hide();
		else
			frame:Show();
		end
	end
end

local function addon_UpdateOrientation(self, orientation)
	-- Update the orientation of the frames.
	-- self == vehicle bar object
	-- orienation == "ACROSS" or "DOWN"

	-- We don't want to rotate the bar while it is being shown on
	-- the vehicle frame.
	local hideVehicle = appliedOptions.vehicleHideFrame;
	if (UnitHasVehicleUI("player")) then
		if (not hideVehicle) then
			-- Frames are on the vehicle.
			-- Do not rotate them.
			return;
		end
	end

	-- Frames are not on a vehicle.
	-- It is ok to rotate them.
	addon_UpdateOrientationNoVehicle(self, orientation);
end

local function addon_Update(self)
	-- Update the frame
	-- self == vehicle bar object

	local hideVehicle = appliedOptions.vehicleHideFrame;
	if (UnitHasVehicleUI("player")) then
		if (not hideVehicle) then
			-- Position the frames on the vehicle.
			addon_UpdateOrientationInVehicle(self, "ACROSS");
			return;
		end
	end

	-- Frames are not on a vehicle.
	-- Position the frames where we want them.

	local vb;
	local height = 30;
	local width = 30;
	local parent = self.helperFrame;
	local hasVehicleUI = UnitHasVehicleUI("player");
	local canExitVehicle = CanExitVehicle();

	local objLeft = VehicleMenuBarLeaveButton;  -- Left most button
	local objRight = VehicleMenuBarLeaveButton;  -- Right most button

	vb = VehicleMenuBarLeaveButton;
	vb:GetNormalTexture():SetTexture([=[Interface\Vehicles\UI-Vehicles-Button-Exit-Up]=])
	vb:GetNormalTexture():SetTexCoord(0.140625, 0.859375, 0.140625, 0.859375)
	vb:GetPushedTexture():SetTexture([=[Interface\Vehicles\UI-Vehicles-Button-Exit-Down]=])
	vb:GetPushedTexture():SetTexCoord(0.140625, 0.859375, 0.140625, 0.859375)
	vb:SetHeight(height);
	vb:SetWidth(width);
	vb:SetParent(self.frame);
	if (canExitVehicle) then
		vb:Show();
	else
		vb:Hide();
	end

	-- IsVehicleAimAngleAdjustable()

	if (appliedOptions.vehicleBarAimButtons) then
		vb = VehicleMenuBarPitchUpButton;
		vb:GetNormalTexture():SetTexture([=[Interface\Vehicles\UI-Vehicles-Button-Pitch-Up]=])
		vb:GetNormalTexture():SetTexCoord(0.21875, 0.765625, 0.234375, 0.78125)
		vb:GetPushedTexture():SetTexture([=[Interface\Vehicles\UI-Vehicles-Button-Pitch-Down]=])
		vb:GetPushedTexture():SetTexCoord(0.21875, 0.765625, 0.234375, 0.78125)
		vb:SetHeight(height);
		vb:SetWidth(width);
		vb:SetParent(self.frame);
		if (hasVehicleUI and VehicleMenuBar.currPitchVisible) then
			vb:Show();
		else
			vb:Hide();
		end

		vb = VehicleMenuBarPitchDownButton;
		vb:GetNormalTexture():SetTexture([=[Interface\Vehicles\UI-Vehicles-Button-PitchDown-Up]=])
		vb:GetNormalTexture():SetTexCoord(0.21875, 0.765625, 0.234375, 0.78125)
		vb:GetPushedTexture():SetTexture([=[Interface\Vehicles\UI-Vehicles-Button-PitchDown-Down]=])
		vb:GetPushedTexture():SetTexCoord(0.21875, 0.765625, 0.234375, 0.78125)
		vb:SetHeight(height);
		vb:SetWidth(width);
		vb:SetParent(self.frame);
		if (hasVehicleUI and VehicleMenuBar.currPitchVisible) then
			vb:Show();
		else
			vb:Hide();
		end

		objRight = VehicleMenuBarPitchDownButton;  -- Right most button
	else
		vb = VehicleMenuBarPitchUpButton;
		vb:Hide();
		vb = VehicleMenuBarPitchDownButton;
		vb:Hide();
	end

	local frame = self.helperFrame;
	frame:ClearAllPoints();
	frame:SetPoint("TOPLEFT", objLeft);
	frame:SetPoint("BOTTOMRIGHT", objRight);

	addon_UpdateOrientationNoVehicle(self, self.orientation);
end

local function addon_Hooked_VehicleMenuBar_SetSkin(skinName, pitchVisible)
	-- (hooksecurefunc of VehicleMenuBar_SetSkin in VehicleMenuBar.lua)

	-- One problem with the way this function works is that if they did not
	-- specify a relative frame in their SkinsData table, then they default to
	-- the button's current parent. This is a problem if we currently have
	-- the button parented to our vehicle bar...the game will stop with an
	-- error.
	--
	-- To avoid this problem we have to unparent the buttons and clear their
	-- point when we get UNIT_ENTERING_VEHICLE and UNIT_EXITING_VEHICLE events.
	-- We can re-establish the values at the end of MainMenuBar_ToVehicleArt
	-- and MainMenuBar_ToPlayerArt because by then Blizzard has finished
	-- trying to position the buttons.

	if (module.ctVehicleBar.isDisabled) then
		return;
	end
	addon_Update(module.ctVehicleBar);
end

local function addon_Hooked_MainMenuBar_ToVehicleArt(self)
	-- (hooksecurefunc of MainMenuBar_ToVehicleArt in MainMenuBar.lua)

	if (module.ctVehicleBar.isDisabled) then
		return;
	end
	addon_Update(module.ctVehicleBar);
end

local function addon_Hooked_MainMenuBar_ToPlayerArt(self)
	-- (hooksecurefunc of MainMenuBar_ToPlayerArt in MainMenuBar.lua)

	if (module.ctVehicleBar.isDisabled) then
		return;
	end
	addon_Update(module.ctVehicleBar);
end

local function addon_ClearVehiclePoints()
	-- Clear the points from the vehicle buttons we are using, and reset the parents to be
	-- the default vehicle menu bar. This is to avoid possible issues if we have the buttons
	-- anchored and parented to our frames when Blizzard tries to position them.
	--
	-- This is the error which occured previously (probably around WoW 3.0 when vehicle
	-- support was first being added to CT_BottomBar):
	--    Framexml\VehicleMenuBar.lua:571: 'VehicleMenuBarLeaveButton:SetPoint():
	--    CT_BottomBar_VehicleBarFrame is dependent on this.'

	if (module.ctVehicleBar.isDisabled) then
		return;
	end
	if (not ctVehicleBar) then
		return;
	end
	local frames = ctVehicleBar.frames;
	if (not frames) then
		return;
	end
	for i = 1, #frames do
		local frame = frames[i];
		if (i ~= 1) then
			frame:SetParent(VehicleMenuBar);
		end
		frame:ClearAllPoints();
	end
end

local function addon_Hooked_VehicleMenuBar_ReleaseSkins()
	if (module.ctVehicleBar.isDisabled) then
		return;
	end
	addon_ClearVehiclePoints();
end

local function addon_Hooked_MainMenuBarVehicleLeaveButton_Update()
	-- (hooksecurefunc of MainMenuBarVehicleLeaveButton_Update in MainMenuBar.lua)
	-- This is the routine where Blizzard shows or hides a vehicle leave button
	-- when the player does not have a vehicle ui.
	if (module.ctVehicleBar.isDisabled) then
		return;
	end
	addon_Update(module.ctVehicleBar);
end

local function addon_OnEvent()
	if (module.ctVehicleBar.isDisabled) then
		return;
	end
	if (event == "UNIT_ENTERING_VEHICLE" or event == "UNIT_EXITING_VEHICLE") then
		addon_ClearVehiclePoints();
	end
end

local function addon_Disable(self)
	addon_UpdateOrientationInVehicle(self, "ACROSS");
end

local function addon_Init(self)
	-- Initialization
	-- self == bags bar object

	appliedOptions = module.appliedOptions;

	module.ctVehicleBar = self;

	local frame = CreateFrame("Frame", "CT_BottomBar_" .. self.frameName .. "_GuideFrame");
	self.helperFrame = frame;

	frame:RegisterEvent("UNIT_ENTERING_VEHICLE");
	frame:RegisterEvent("UNIT_EXITING_VEHICLE");
	frame:SetScript("OnEvent", addon_OnEvent);

	-- We can't create our own buttons for the pitch up and pitch down functions
	-- because the functions are only available to the Blizzard UI. We need to move
	-- Blizzard's buttons onto our frame when they are not visible on the vehicle frame.
	--
	-- The restricted functions are: VehicleAimUpStart, VehicleAimUpStop,
	-- VehicleAimDownStart, and VehicleAimDownStop.
	--
	-- The buttons are not secure so we can reposition them during combat.

	hooksecurefunc("VehicleMenuBar_SetSkin", addon_Hooked_VehicleMenuBar_SetSkin);
	hooksecurefunc("MainMenuBar_ToVehicleArt", addon_Hooked_MainMenuBar_ToVehicleArt);
	hooksecurefunc("MainMenuBar_ToPlayerArt", addon_Hooked_MainMenuBar_ToPlayerArt);
	hooksecurefunc("VehicleMenuBar_ReleaseSkins", addon_Hooked_VehicleMenuBar_ReleaseSkins);
	hooksecurefunc("MainMenuBarVehicleLeaveButton_Update", addon_Hooked_MainMenuBarVehicleLeaveButton_Update);
end

local function addon_Register()
	module:registerAddon(
		"Vehicle Bar",  -- option name
		"VehicleToolsBar",  -- used in frame names
		"Vehicle Bar",  -- shown in options window & tooltips
		"Vehicle",  -- title for horizontal orientation
		"Vehicle",  -- title for vertical orientation
		{ "BOTTOMLEFT", ctRelativeFrame, "BOTTOM", 269, 107 },
		{ -- settings
			usedOnVehicle = true,
			orientation = "ACROSS",
		},
		addon_Init,
		nil,  -- no post init function
		addon_Update,
		addon_UpdateOrientation,
		nil,  -- no enable function
		addon_Disable,
		"helperFrame",
		VehicleMenuBarLeaveButton,
		VehicleMenuBarPitchUpButton,
		VehicleMenuBarPitchDownButton
	);
end

module.loadedAddons["Vehicle Bar"] = addon_Register;
